/* -*- c++ -*- */
/* 
 * Copyright 2019 Thomas Kolb.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_HAMNET70_CORRECT_PHASE_FROM_TAG_H
#define INCLUDED_HAMNET70_CORRECT_PHASE_FROM_TAG_H

#include <hamnet70/api.h>
#include <gnuradio/sync_block.h>

namespace gr {
  namespace hamnet70 {

    /*!
     * \brief Rotate the phase by the value specified in the given tag.
     * \ingroup hamnet70
     *
     */
    class HAMNET70_API correct_phase_from_tag : virtual public gr::sync_block
    {
     public:
      typedef boost::shared_ptr<correct_phase_from_tag> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of hamnet70::correct_phase_from_tag.
       *
       * To avoid accidental use of raw pointers, hamnet70::correct_phase_from_tag's
       * constructor is in a private implementation
       * class. hamnet70::correct_phase_from_tag::make is the public interface for
       * creating new instances.
       */
      static sptr make(const std::string &phase_tag_name = "phase_est");
    };

  } // namespace hamnet70
} // namespace gr

#endif /* INCLUDED_HAMNET70_CORRECT_PHASE_FROM_TAG_H */

