/* -*- c++ -*- */
/* 
 * Copyright 2019 Thomas Kolb.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_HAMNET70_CORRECT_FREQUENCY_IMPL_H
#define INCLUDED_HAMNET70_CORRECT_FREQUENCY_IMPL_H

#include <hamnet70/correct_frequency.h>

namespace gr {
	namespace hamnet70 {

		class correct_frequency_impl : public correct_frequency
		{
			private:
				std::vector<gr_complex> d_symbols;
				gr_complex    d_correctionVector;
				gr_complex    d_correctionIncrement;

				float         d_lastPhase;
				float         d_filteredPhaseInc;
				float         d_unwrapOffset;
				size_t        d_preIdx;

				static const constexpr float c_phaseInc_alpha = 0.001f;

			public:
				correct_frequency_impl(const std::vector<gr_complex> &symbols);
				~correct_frequency_impl();

				// Where all the action really happens
				int work(int noutput_items,
						gr_vector_const_void_star &input_items,
						gr_vector_void_star &output_items);
		};

	} // namespace hamnet70
} // namespace gr

#endif /* INCLUDED_HAMNET70_CORRECT_FREQUENCY_IMPL_H */

